# coding=utf-8
# Copyright 2023 HuggingFace Inc. team and MosaicML NLP team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PyTorch MPT model."""

import math
from typing import Optional, Tuple, Union

import torch
import torch.utils.checkpoint
from torch import nn
from torch.nn import BCEWithLogitsLoss, CrossEntropyLoss, LayerNorm, MSELoss
from torch.nn import functional as F

from ...file_utils import add_code_sample_docstrings, add_start_docstrings, add_start_docstrings_to_model_forward
from ...modeling_outputs import (
    BaseModelOutputWithPastAndCrossAttentions,
    CausalLMOutputWithCrossAttentions,
    QuestionAnsweringModelOutput,
    SequenceClassifierOutputWithPast,
    TokenClassifierOutput,
)
from ...modeling_utils import PreTrainedModel
from ...utils import logging
from .configuration_mpt import MptConfig


logger = logging.get_logger(__name__)

_CHECKPOINT_FOR_DOC = "mosaicml/mpt-7b"
_CONFIG_FOR_DOC = "MptConfig"

MPT_PRETRAINED_MODEL_ARCHIVE_LIST = [
    "mosaicml/mpt-7b",
    "mosaicml/mpt-7b-storywriter",
    "mosaicml/mpt-7b-instruct",
    "mosaicml/mpt-7b-8k",
    "mosaicml/mpt-7b-8k-instruct",
    "mosaicml/mpt-7b-8k-chat",
    "mosaicml/mpt-30b",
    "mosaicml/mpt-30b-instruct",
    "mosaicml/mpt-30b-chat"
    # See all MPT models at https://huggingface.co/models?filter=mpt
]


# Copied from transformers.models.bloom.modeling_bloom._make_causal_mask
def _make_causal_mask(
    input_ids_shape: torch.Size, device: torch.device, past_key_values_length: int
) -> torch.BoolTensor:
    """
    Make causal mask used for self-attention.
    """
    batch_size, target_length = input_ids_shape
    mask = torch.empty((target_length, target_length + past_key_values_length), dtype=torch.bool, device=device)
    # ONNX doesn't support `torch.Tensor.triu` properly, thus we use this workaround
    seq_ids = torch.arange(target_length, device=device)
    mask[:, past_key_values_length:] = seq_ids[:, None] < seq_ids[None, :]

    if past_key_values_length > 0:
        mask[:, :past_key_values_length] = False

    expanded_mask = mask[None, None, :, :].expand(batch_size, 1, target_length, target_length + past_key_values_length)
    return expanded_mask


# Copied from transformers.models.bloom.modeling_bloom._expand_mask
def _expand_mask(mask: torch.Tensor, tgt_length: int) -> torch.BoolTensor:
    """
    Expands attention_mask from `[batch_size, src_length]` to `[batch_size, 1, tgt_length, src_length]`.
    """
    batch_size, src_length = mask.shape
    tgt_length = tgt_length if tgt_length is not None else src_length

    expanded_mask = ~(mask[:, None, None, :].to(torch.bool))
    return expanded_mask.expand(batch_size, 1, tgt_length, src_length)


def build_mpt_alibi_tensor(num_heads, sequence_length, alibi_bias_max=8, device=None):
    r"""
    Link to paper: https://arxiv.org/abs/2108.12409 - Alibi tensor is not causal as the original paper mentions, it
    relies on a translation invariance of softmax for quick implementation. This implementation has been copied from
    the alibi implementation of MPT source code that led to slightly different results than the Bloom alibi:
    https://huggingface.co/mosaicml/mpt-7b/blob/main/attention.py#L292
    """
    alibi = torch.arange(1 - sequence_length, 1, dtype=torch.int32, device=device).view(1, 1, 1, sequence_length)
    num_heads_power_of_2 = 2 ** math.ceil(math.log2(num_heads))

    base = torch.arange(1, num_heads_power_of_2 + 1, dtype=torch.float32, device=device)
    base = base * (alibi_bias_max / num_heads_power_of_2)

    slopes = 1.0 / torch.pow(2, base)
    slopes = slopes.view(1, num_heads, 1, 1)

    if num_heads_power_of_2 != num_heads:
        slopes = torch.concat([slopes[1::2], slopes[::2]])[:num_heads]

    alibi = alibi * slopes
    return alibi.squeeze(0)


class MptAttention(nn.Module):
    """Multi-head self attention.
    Using torch or triton attention implemetation enables user to also use additive bias.
    """

    def __init__(self, config: MptConfig):
        super().__init__()
        self.hidden_size = config.hidden_size
        self.n_heads = config.n_heads
        self.max_seq_length = config.max_seq_len
        self.head_dim = self.hidden_size // self.n_heads
        self.softmax_scale = config.attn_config.softmax_scale
        if self.softmax_scale is None:
            self.softmax_scale = 1 / math.sqrt(self.hidden_size / self.n_heads)

        self.attn_dropout_p = config.attn_config.attn_pdrop
        self.Wqkv = nn.Linear(self.hidden_size, 3 * self.hidden_size, bias=False)
        self.out_proj = nn.Linear(self.hidden_size, self.hidden_size, bias=False)

    def forward(
        self,
        hidden_states: torch.Tensor,
        position_bias: torch.Tensor,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        batch_size, seq_length = hidden_states.shape[:2]

        mixed_qkv = self.Wqkv(hidden_states)
        query_states, key_states, value_states = mixed_qkv.chunk(3, dim=2)
        query_states = query_states.reshape(batch_size, seq_length, self.n_heads, self.head_dim).transpose(1, 2)
        key_states = key_states.reshape(batch_size, seq_length, self.n_heads, self.head_dim).transpose(1, 2)
        value_states = value_states.reshape(batch_size, seq_length, self.n_heads, self.head_dim).transpose(1, 2)

        if past_key_value is not None:
            if len(past_key_value) != 0:
                key_states = torch.cat([past_key_value[0], key_states], dim=2)
                value_states = torch.cat([past_key_value[1], value_states], dim=2)
            past_key_value = (key_states, value_states)
        else:
            past_key_value = (key_states, value_states)

        attention_scores = torch.matmul(query_states, key_states.transpose(-1, -2)) * self.softmax_scale

        query_length = seq_length if past_key_value is None else seq_length + past_key_value[0].shape[2]

        if position_bias is not None:
            if len(position_bias.shape) != 3:
                raise ValueError(f"Expecting position_bias shape to be 3 dimensions, got {len(position_bias.shape)}")
            key_length = key_states.shape[-2]

            position_bias_query_index = max(0, position_bias.size(1) - query_length)
            position_bias_key_index = max(0, position_bias.size(2) - key_length)

            position_bias = position_bias[:, position_bias_query_index:, position_bias_key_index:]

            attention_scores = attention_scores + position_bias

        if attention_mask is not None:
            attention_scores = attention_scores.masked_fill(attention_mask, torch.finfo(query_states.dtype).min)

        # (batch_size, n_heads, seq_length, key_length)
        attn_weights = nn.functional.softmax(attention_scores.float(), dim=-1).to(value_states.dtype)
        attn_weights = nn.functional.dropout(attn_weights, p=self.attn_dropout_p, training=self.training)

        context_states = torch.matmul(attn_weights, value_states)
        context_states = context_states.permute(0, 2, 1, 3).contiguous().view(batch_size, seq_length, -1)
        attn_output = self.out_proj(context_states)

        return attn_output, attn_weights, past_key_value


class MptMLP(nn.Module):
    def __init__(self, config: MptConfig):
        super().__init__()
        hidden_size = config.hidden_size

        self.up_proj = nn.Linear(hidden_size, 4 * hidden_size, bias=False)
        self.act = nn.GELU(approximate="none")
        self.down_proj = nn.Linear(4 * hidden_size, hidden_size, bias=False)
        self.hidden_dropout = config.attn_config.attn_pdrop

    def forward(self, hidden_states: torch.Tensor, residual: torch.Tensor) -> torch.Tensor:
        hidden_states = self.act(self.up_proj(hidden_states))

        intermediate_output = self.down_proj(hidden_states)

        output = F.dropout(intermediate_output, p=self.hidden_dropout, training=self.training)
        output = output + residual

        return output


class MptBlock(nn.Module):
    def __init__(self, config: MptConfig):
        super().__init__()
        hidden_size = config.hidden_size

        self.norm_1 = LayerNorm(hidden_size, eps=config.layer_norm_epsilon)
        # backward compatibility with weights on the Hub
        self.norm_1.bias = None

        self.num_heads = config.n_heads
        self.attn = MptAttention(config)

        self.norm_2 = LayerNorm(hidden_size, eps=config.layer_norm_epsilon)
        # backward compatibility with weights on the Hub
        self.norm_2.bias = None

        self.ffn = MptMLP(config)

        self.dropout_rate = config.attn_config.attn_pdrop
        self.resid_attn_dropout = nn.Dropout(self.dropout_rate)

    def forward(
        self,
        hidden_states: torch.Tensor,
        position_bias: torch.Tensor,
        attention_mask: torch.Tensor,
        layer_past: Optional[Tuple[torch.Tensor, torch.Tensor]] = None,
        use_cache: bool = False,
        output_attentions: bool = False,
    ):
        # hidden_states: [batch_size, seq_length, hidden_size]
        # Layer norm at the beginning of the transformer layer.
        layernorm_output = self.norm_1(hidden_states)

        residual = hidden_states

        # Self attention.
        attn_outputs, attn_weights, past_key_value = self.attn(
            layernorm_output,
            position_bias=position_bias,
            attention_mask=attention_mask,
            past_key_value=layer_past,
        )

        hidden_states = self.resid_attn_dropout(attn_outputs) + residual

        layernorm_output = self.norm_2(hidden_states)

        # Get residual
        residual = hidden_states

        # MLP.
        output = self.ffn(layernorm_output, residual)
        outputs = (output,)

        if use_cache:
            outputs += (past_key_value,)

        if output_attentions:
            outputs += (attn_weights,)

        return outputs  # hidden_states, present, attentions


class MptPreTrainedModel(PreTrainedModel):
    config_class = MptConfig
    base_model_prefix = "transformer"
    supports_gradient_checkpointing = True
    _no_split_modules = ["MptBlock"]
    _keys_to_ignore_on_load_missing = [r"lm_head.*."]

    def __init__(self, *inputs, **kwargs):
        super().__init__(*inputs, **kwargs)

    def _init_weights(self, module: nn.Module):
        """Initialize the weights."""
        if isinstance(module, nn.Linear):
            # Slightly different from the TF version which uses truncated_normal for initialization
            # cf https://github.com/pytorch/pytorch/pull/5617
            module.weight.data.normal_(mean=0.0, std=self.config.initializer_range)
            if module.bias is not None:
                module.bias.data.zero_()
        elif isinstance(module, nn.Embedding):
            module.weight.data.normal_(mean=0.0, std=self.config.initializer_range)
            if module.padding_idx is not None:
                module.weight.data[module.padding_idx].zero_()
        elif isinstance(module, LayerNorm):
            if module.bias is not None:
                module.bias.data.zero_()
            module.weight.data.fill_(1.0)

    def _set_gradient_checkpointing(self, module: nn.Module, value: bool = False):
        if isinstance(module, MptModel):
            module.gradient_checkpointing = value

    @staticmethod
    def _convert_to_mpt_cache(
        past_key_value: Tuple[Tuple[torch.Tensor, torch.Tensor]]
    ) -> Tuple[Tuple[torch.Tensor, torch.Tensor]]:
        """
        Converts the cache to the format expected by Mpt, i.e. to tuple(tuple([batch_size * num_heads, ...]))
        """
        batch_size, num_heads, head_dim, seq_length = past_key_value[0][0].shape
        batch_size_times_num_heads = batch_size * num_heads
        # key:  [batch_size, num_heads, head_dim, seq_length] -> [batch_size * num_heads, head_dim, seq_length]
        # value: [batch_size, num_heads, seq_length, head_dim] -> [batch_size * num_heads, seq_length, head_dim]
        return tuple(
            (
                layer_past[0].reshape(batch_size_times_num_heads, head_dim, seq_length),
                layer_past[1].reshape(batch_size_times_num_heads, seq_length, head_dim),
            )
            for layer_past in past_key_value
        )


MPT_START_DOCSTRING = r"""

    This model inherits from [`PreTrainedModel`]. Check the superclass documentation for the generic methods the
    library implements for all its model (such as downloading or saving, resizing the input embeddings etc.)

    This model is also a PyTorch [torch.nn.Module](https://pytorch.org/docs/stable/nn.html#torch.nn.Module) subclass.
    Use it as a regular PyTorch Module and refer to the PyTorch documentation for all matter related to general usage
    and behavior.

    Parameters:
        config ([`MptConfig`]): Model configuration class with all the parameters of the model.
            Initializing with a config file does not load the weights associated with the model, only the
            configuration. Check out the [`~PreTrainedModel.from_pretrained`] method to load the model weights.
"""

MPT_INPUTS_DOCSTRING = r"""
    Args:
        input_ids (`torch.LongTensor` of shape `(batch_size, input_ids_length)`):
            `input_ids_length` = `sequence_length` if `past_key_values` is `None` else `past_key_values[0][0].shape[2]`
            (`sequence_length` of input past key value states). Indices of input sequence tokens in the vocabulary.

            If `past_key_values` is used, only `input_ids` that do not have their past calculated should be passed as
            `input_ids`.

            Indices can be obtained using [`AutoTokenizer`]. See [`PreTrainedTokenizer.encode`] and
            [`PreTrainedTokenizer.__call__`] for details.

            [What are input IDs?](../glossary#input-ids)
        past_key_values (`Tuple[Tuple[torch.Tensor]]` of length `config.n_layers`):
            Contains precomputed hidden-states (key and values in the attention blocks) as computed by the model (see
            `past_key_values` output below). Can be used to speed up sequential decoding. The `input_ids` which have
            their past given to this model should not be passed as `input_ids` as they have already been computed.

            Each element of `past_key_values` is a tuple (past_key, past_value):
            - past_key: [batch_size * num_heads, head_dim, kv_length]
            - past_value: [batch_size * num_heads, kv_length, head_dim]
        attention_mask (`torch.FloatTensor` of shape `(batch_size, sequence_length)`, *optional*):
            Mask to avoid performing attention on padding token indices. Mask values selected in `[0, 1]`:

            - 1 for tokens that are **not masked**,
            - 0 for tokens that are **masked**.

            [What are attention masks?](../glossary#attention-mask)

        inputs_embeds (`torch.FloatTensor` of shape `(batch_size, sequence_length, hidden_size)`, *optional*):
            Optionally, instead of passing `input_ids` you can choose to directly pass an embedded representation. This
            is useful if you want more control over how to convert `input_ids` indices into associated vectors than the
            model's internal embedding lookup matrix.

            If `past_key_values` is used, optionally only the last `inputs_embeds` have to be input (see
            `past_key_values`).
        use_cache (`bool`, *optional*):
            If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding (see
            `past_key_values`).
        output_attentions (`bool`, *optional*):
            Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
            tensors for more detail.
        output_hidden_states (`bool`, *optional*):
            Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
            more detail.
        return_dict (`bool`, *optional*):
            Whether or not to return a [`~file_utils.ModelOutput`] instead of a plain tuple.
"""


@add_start_docstrings(
    "The bare Mpt Model transformer outputting raw hidden-states without any specific head on top.",
    MPT_START_DOCSTRING,
)
class MptModel(MptPreTrainedModel):
    def __init__(self, config: MptConfig):
        super().__init__(config)

        self.hidden_size = config.hidden_size
        self.num_heads = config.n_heads

        # Embedding + LN Embedding
        self.wte = nn.Embedding(config.vocab_size, self.hidden_size)

        # Transformer blocks
        self.blocks = nn.ModuleList([MptBlock(config) for _ in range(config.n_layers)])

        # Final Layer Norm
        self.norm_f = LayerNorm(self.hidden_size, eps=config.layer_norm_epsilon)
        # backward compatibility with weights on the Hub
        self.norm_f.bias = None

        self.gradient_checkpointing = False

        # Initialize weights and apply final processing
        self.post_init()

    def get_input_embeddings(self):
        return self.wte

    def build_mpt_alibi_tensor(self, num_heads, sequence_length, alibi_bias_max=8, device=None):
        return build_mpt_alibi_tensor(num_heads, sequence_length, alibi_bias_max, device)

    def _prepare_attn_mask(
        self, attention_mask: torch.Tensor, input_shape: Tuple[int, int], past_key_values_length: int
    ) -> torch.BoolTensor:
        # create causal mask
        # [batch_size, seq_length] -> [batch_size, 1, tgt_length, src_length]
        if input_shape[1] + past_key_values_length != attention_mask.shape[1]:
            raise ValueError(
                "Attention mask shape should be (batch_size, seq_length + past_key_values_length)"
                f" but is {attention_mask.shape} with input_ids shape {input_shape} and past length"
                f" {past_key_values_length}."
            )
        combined_attention_mask = None
        device = attention_mask.device
        _, src_length = input_shape

        if src_length > 1:
            combined_attention_mask = _make_causal_mask(
                input_shape, device=device, past_key_values_length=past_key_values_length
            )

        # [batch_size, seq_length] -> [batch_size, 1, tgt_length, src_length]
        expanded_attn_mask = _expand_mask(attention_mask, tgt_length=src_length)
        combined_attention_mask = (
            expanded_attn_mask if combined_attention_mask is None else expanded_attn_mask | combined_attention_mask
        )

        return combined_attention_mask

    def set_input_embeddings(self, new_embeddings: torch.Tensor):
        self.wte = new_embeddings

    @add_start_docstrings_to_model_forward(MPT_INPUTS_DOCSTRING)
    @add_code_sample_docstrings(
        checkpoint=_CHECKPOINT_FOR_DOC,
        output_type=BaseModelOutputWithPastAndCrossAttentions,
        config_class=_CONFIG_FOR_DOC,
    )
    def forward(
        self,
        input_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[Tuple[Tuple[torch.Tensor, torch.Tensor], ...]] = None,
        attention_mask: Optional[torch.Tensor] = None,
        inputs_embeds: Optional[torch.LongTensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple[torch.Tensor, ...], BaseModelOutputWithPastAndCrossAttentions]:
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        use_cache = use_cache if use_cache is not None else self.config.use_cache
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        if input_ids is not None and inputs_embeds is not None:
            raise ValueError("You cannot specify both input_ids and inputs_embeds at the same time")
        elif input_ids is not None:
            batch_size, seq_length = input_ids.shape
        elif inputs_embeds is not None:
            batch_size, seq_length, _ = inputs_embeds.shape
        else:
            raise ValueError("You have to specify either input_ids or inputs_embeds")

        if past_key_values is None:
            past_key_values = tuple([None] * len(self.blocks))

        if inputs_embeds is None:
            inputs_embeds = self.wte(input_ids)

        hidden_states = inputs_embeds

        presents = () if use_cache else None
        all_self_attentions = () if output_attentions else None
        all_hidden_states = () if output_hidden_states else None

        if self.gradient_checkpointing and self.training:
            if use_cache:
                logger.warning_once(
                    "`use_cache=True` is incompatible with gradient checkpointing. Setting `use_cache=False`..."
                )
                use_cache = False

        # Compute alibi tensor: check build_alibi_tensor documentation
        seq_length_with_past = seq_length
        past_key_values_length = 0
        if past_key_values[0] is not None:
            past_key_values_length = past_key_values[0][0].shape[2]
            seq_length_with_past = seq_length_with_past + past_key_values_length
        if attention_mask is None:
            attention_mask = torch.ones((batch_size, seq_length_with_past), device=hidden_states.device)
        else:
            attention_mask = attention_mask.to(hidden_states.device)

        alibi = self.build_mpt_alibi_tensor(self.num_heads, self.config.max_seq_len, device=hidden_states.device)

        causal_mask = self._prepare_attn_mask(
            attention_mask,
            input_shape=(batch_size, seq_length),
            past_key_values_length=past_key_values_length,
        )

        for i, (block, layer_past) in enumerate(zip(self.blocks, past_key_values)):
            if output_hidden_states:
                all_hidden_states = all_hidden_states + (hidden_states,)

            if self.gradient_checkpointing and self.training:

                def create_custom_forward(module):
                    def custom_forward(*inputs):
                        # None for past_key_value
                        return module(*inputs, use_cache=use_cache, output_attentions=output_attentions)

                    return custom_forward

                outputs = torch.utils.checkpoint.checkpoint(
                    create_custom_forward(block),
                    hidden_states,
                    alibi,
                    causal_mask,
                    layer_past,
                )
            else:
                outputs = block(
                    hidden_states,
                    layer_past=layer_past,
                    attention_mask=causal_mask,
                    use_cache=use_cache,
                    output_attentions=output_attentions,
                    position_bias=alibi,
                )

            hidden_states = outputs[0]
            if use_cache is True:
                presents = presents + (outputs[1],)

            if output_attentions:
                all_self_attentions = all_self_attentions + (outputs[2 if use_cache else 1],)

        # Add last hidden state
        hidden_states = self.norm_f(hidden_states)

        if output_hidden_states:
            all_hidden_states = all_hidden_states + (hidden_states,)

        if not return_dict:
            return tuple(v for v in [hidden_states, presents, all_hidden_states, all_self_attentions] if v is not None)

        return BaseModelOutputWithPastAndCrossAttentions(
            last_hidden_state=hidden_states,
            past_key_values=presents,
            hidden_states=all_hidden_states,
            attentions=all_self_attentions,
        )


@add_start_docstrings(
    """
    The MPT Model transformer with a language modeling head on top (linear layer with weights tied to the input
    embeddings).
    """,
    MPT_START_DOCSTRING,
)
class MptForCausalLM(MptPreTrainedModel):
    _tied_weights_keys = ["lm_head.weight"]

    def __init__(self, config: MptConfig):
        super().__init__(config)
        self.transformer = MptModel(config)
        self.lm_head = nn.Linear(config.hidden_size, config.vocab_size, bias=False)

        # Initialize weights and apply final processing
        self.post_init()

    def get_output_embeddings(self):
        return self.lm_head

    def set_output_embeddings(self, new_embeddings: torch.Tensor):
        self.lm_head = new_embeddings

    def prepare_inputs_for_generation(
        self,
        input_ids: torch.LongTensor,
        past_key_values: Optional[torch.Tensor] = None,
        attention_mask: Optional[torch.Tensor] = None,
        inputs_embeds: Optional[torch.Tensor] = None,
        use_cache: Optional[bool] = None,
        **kwargs,
    ) -> dict:
        # only last token for input_ids if past is not None
        if past_key_values:
            input_ids = input_ids[:, -1].unsqueeze(-1)

        # if `inputs_embeds` are passed, we only want to use them in the 1st generation step
        if inputs_embeds is not None and past_key_values is None:
            model_inputs = {"inputs_embeds": inputs_embeds}
        else:
            model_inputs = {"input_ids": input_ids}

        model_inputs.update(
            {
                "past_key_values": past_key_values,  # NITS should it be layer_past?
                "use_cache": use_cache,
                "attention_mask": attention_mask,
            }
        )
        return model_inputs

    @add_start_docstrings_to_model_forward(MPT_INPUTS_DOCSTRING)
    @add_code_sample_docstrings(
        checkpoint=_CHECKPOINT_FOR_DOC,
        output_type=CausalLMOutputWithCrossAttentions,
        config_class=_CONFIG_FOR_DOC,
    )
    def forward(
        self,
        input_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[Tuple[Tuple[torch.Tensor, torch.Tensor], ...]] = None,
        attention_mask: Optional[torch.Tensor] = None,
        inputs_embeds: Optional[torch.Tensor] = None,
        labels: Optional[torch.Tensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple[torch.Tensor], CausalLMOutputWithCrossAttentions]:
        r"""
        labels (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
            Labels for language modeling. Note that the labels **are shifted** inside the model, i.e. you can set
            `labels = input_ids` Indices are selected in `[-100, 0, ..., config.vocab_size]` All labels set to `-100`
            are ignored (masked), the loss is only computed for labels in `[0, ..., config.vocab_size]`
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        transformer_outputs = self.transformer(
            input_ids,
            past_key_values=past_key_values,
            attention_mask=attention_mask,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        hidden_states = transformer_outputs[0]

        lm_logits = self.lm_head(hidden_states)

        loss = None
        if labels is not None:
            # move labels to correct device to enable model parallelism
            labels = labels.to(lm_logits.device)
            # Shift so that tokens < n predict n
            shift_logits = lm_logits[..., :-1, :].contiguous()
            shift_labels = labels[..., 1:].contiguous()
            batch_size, seq_length, vocab_size = shift_logits.shape
            # Flatten the tokens
            loss_fct = CrossEntropyLoss()
            loss = loss_fct(
                shift_logits.view(batch_size * seq_length, vocab_size), shift_labels.view(batch_size * seq_length)
            )

        if not return_dict:
            output = (lm_logits,) + transformer_outputs[1:]
            return ((loss,) + output) if loss is not None else output

        return CausalLMOutputWithCrossAttentions(
            loss=loss,
            logits=lm_logits,
            past_key_values=transformer_outputs.past_key_values,
            hidden_states=transformer_outputs.hidden_states,
            attentions=transformer_outputs.attentions,
        )

    def _reorder_cache(
        self, past: Tuple[Tuple[torch.Tensor, torch.Tensor], ...], beam_idx: torch.LongTensor
    ) -> Tuple[Tuple[torch.Tensor, torch.Tensor], ...]:
        """
        This function is used to re-order the `past_key_values` cache if [`~PreTrainedModel.beam_search`] or
        [`~PreTrainedModel.beam_sample`] is called. This is required to match `past_key_values` with the correct
        beam_idx at every generation step.

        Output shares the same memory storage as `past`.
        """
        # Get a copy of `beam_idx` on all the devices where we need those indices.
        device_to_beam_idx = {
            past_state.device: beam_idx.to(past_state.device) for layer_past in past for past_state in layer_past
        }
        reordered_past = tuple(
            (
                layer_past[0].index_select(0, device_to_beam_idx[layer_past[0].device]),
                layer_past[1].index_select(0, device_to_beam_idx[layer_past[0].device]),
            )
            for layer_past in past
        )
        return reordered_past


@add_start_docstrings(
    """
    The MPT Model transformer with a sequence classification head on top (linear layer).

    [`MptForSequenceClassification`] uses the last token in order to do the classification, as other causal models
    (e.g. GPT-1) do.

    Since it does classification on the last token, it requires to know the position of the last token. If a
    `pad_token_id` is defined in the configuration, it finds the last token that is not a padding token in each row. If
    no `pad_token_id` is defined, it simply takes the last value in each row of the batch. Since it cannot guess the
    padding tokens when `inputs_embeds` are passed instead of `input_ids`, it does the same (take the last value in
    each row of the batch).
    """,
    MPT_START_DOCSTRING,
)
class MptForSequenceClassification(MptPreTrainedModel):
    def __init__(self, config: MptConfig):
        super().__init__(config)
        self.num_labels = config.num_labels
        self.transformer = MptModel(config)
        self.score = nn.Linear(config.hidden_size, config.num_labels, bias=False)

        # Initialize weights and apply final processing
        self.post_init()

    @add_start_docstrings_to_model_forward(MPT_INPUTS_DOCSTRING)
    @add_code_sample_docstrings(
        checkpoint=_CHECKPOINT_FOR_DOC,
        output_type=SequenceClassifierOutputWithPast,
        config_class=_CONFIG_FOR_DOC,
    )
    def forward(
        self,
        input_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[Tuple[Tuple[torch.Tensor, torch.Tensor], ...]] = None,
        attention_mask: Optional[torch.Tensor] = None,
        inputs_embeds: Optional[torch.Tensor] = None,
        labels: Optional[torch.Tensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple[torch.Tensor], SequenceClassifierOutputWithPast]:
        r"""
        labels (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
            Labels for computing the sequence classification/regression loss. Indices should be in `[0, ...,
            config.num_labels - 1]`. If `config.num_labels == 1` a regression loss is computed (Mean-Square loss), If
            `config.num_labels > 1` a classification loss is computed (Cross-Entropy).
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        transformer_outputs = self.transformer(
            input_ids,
            past_key_values=past_key_values,
            attention_mask=attention_mask,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )

        hidden_states = transformer_outputs[0]
        logits = self.score(hidden_states)

        if input_ids is not None:
            batch_size = input_ids.shape[0]
        else:
            batch_size = inputs_embeds.shape[0]

        if self.config.pad_token_id is None and batch_size != 1:
            raise ValueError("Cannot handle batch sizes > 1 if no padding token is defined.")
        if self.config.pad_token_id is None:
            sequence_lengths = -1
        else:
            if input_ids is not None:
                sequence_lengths = (torch.ne(input_ids, self.config.pad_token_id).sum(-1) - 1).to(logits.device)
            else:
                sequence_lengths = -1
                logger.warning(
                    f"{self.__class__.__name__} will not detect padding tokens in `inputs_embeds`. Results may be "
                    "unexpected if using padding tokens in conjunction with `inputs_embeds.`"
                )

        pooled_logits = logits[torch.arange(batch_size, device=logits.device), sequence_lengths]

        loss = None
        if labels is not None:
            if self.config.problem_type is None:
                if self.num_labels == 1:
                    self.config.problem_type = "regression"
                elif self.num_labels > 1 and (labels.dtype == torch.long or labels.dtype == torch.int):
                    self.config.problem_type = "single_label_classification"
                else:
                    self.config.problem_type = "multi_label_classification"

            if self.config.problem_type == "regression":
                loss_fct = MSELoss()
                if self.num_labels == 1:
                    loss = loss_fct(pooled_logits.squeeze(), labels.squeeze())
                else:
                    loss = loss_fct(pooled_logits, labels)
            elif self.config.problem_type == "single_label_classification":
                loss_fct = CrossEntropyLoss()
                loss = loss_fct(pooled_logits, labels)
            elif self.config.problem_type == "multi_label_classification":
                loss_fct = BCEWithLogitsLoss()
                loss = loss_fct(pooled_logits, labels)
        if not return_dict:
            output = (pooled_logits,) + transformer_outputs[1:]
            return ((loss,) + output) if loss is not None else output

        return SequenceClassifierOutputWithPast(
            loss=loss,
            logits=pooled_logits,
            past_key_values=transformer_outputs.past_key_values,
            hidden_states=transformer_outputs.hidden_states,
            attentions=transformer_outputs.attentions,
        )


@add_start_docstrings(
    """
    MPT Model with a token classification head on top (a linear layer on top of the hidden-states output) e.g. for
    Named-Entity-Recognition (NER) tasks.
    """,
    MPT_START_DOCSTRING,
)
class MptForTokenClassification(MptPreTrainedModel):
    def __init__(self, config: MptConfig):
        super().__init__(config)
        self.num_labels = config.num_labels

        self.transformer = MptModel(config)
        if hasattr(config, "classifier_dropout") and config.classifier_dropout is not None:
            classifier_dropout = config.classifier_dropout
        elif hasattr(config, "hidden_dropout") and config.hidden_dropout is not None:
            classifier_dropout = config.hidden_dropout
        else:
            classifier_dropout = 0.1
        self.dropout = nn.Dropout(classifier_dropout)
        self.classifier = nn.Linear(config.hidden_size, config.num_labels)

        # Initialize weights and apply final processing
        self.post_init()

    @add_start_docstrings_to_model_forward(MPT_INPUTS_DOCSTRING)
    @add_code_sample_docstrings(
        checkpoint=_CHECKPOINT_FOR_DOC,
        output_type=TokenClassifierOutput,
        config_class=_CONFIG_FOR_DOC,
    )
    def forward(
        self,
        input_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[Tuple[Tuple[torch.Tensor, torch.Tensor], ...]] = None,
        attention_mask: Optional[torch.Tensor] = None,
        inputs_embeds: Optional[torch.Tensor] = None,
        labels: Optional[torch.Tensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        **deprecated_arguments,
    ) -> Union[Tuple[torch.Tensor], TokenClassifierOutput]:
        r"""
        labels (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
            Labels for computing the sequence classification/regression loss. Indices should be in `[0, ...,
            config.num_labels - 1]`. If `config.num_labels == 1` a regression loss is computed (Mean-Square loss), If
            `config.num_labels > 1` a classification loss is computed (Cross-Entropy).
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        transformer_outputs = self.transformer(
            input_ids,
            past_key_values=past_key_values,
            attention_mask=attention_mask,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )

        hidden_states = transformer_outputs[0]
        hidden_states = self.dropout(hidden_states)
        logits = self.classifier(hidden_states)

        loss = None
        if labels is not None:
            # move labels to correct device to enable model parallelism
            labels = labels.to(logits.device)
            batch_size, seq_length = labels.shape
            loss_fct = CrossEntropyLoss()
            loss = loss_fct(
                logits.view(batch_size * seq_length, self.num_labels), labels.view(batch_size * seq_length)
            )

        if not return_dict:
            output = (logits,) + transformer_outputs[2:]
            return ((loss,) + output) if loss is not None else output

        return TokenClassifierOutput(
            loss=loss,
            logits=logits,
            hidden_states=transformer_outputs.hidden_states,
            attentions=transformer_outputs.attentions,
        )


@add_start_docstrings(
    """
    The MPT Model transformer with a span classification head on top for extractive question-answering tasks like SQuAD
    (a linear layers on top of the hidden-states output to compute `span start logits` and `span end logits`).
    """,
    MPT_START_DOCSTRING,
)
class MptForQuestionAnswering(MptPreTrainedModel):
    def __init__(self, config):
        super().__init__(config)
        self.transformer = MptModel(config)
        self.qa_outputs = nn.Linear(config.hidden_size, 2)

        # Initialize weights and apply final processing
        self.post_init()

    @add_start_docstrings_to_model_forward(MPT_INPUTS_DOCSTRING.format("batch_size, sequence_length"))
    def forward(
        self,
        input_ids: Optional[torch.LongTensor] = None,
        attention_mask: Optional[torch.FloatTensor] = None,
        inputs_embeds: Optional[torch.FloatTensor] = None,
        start_positions: Optional[torch.LongTensor] = None,
        end_positions: Optional[torch.LongTensor] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple, QuestionAnsweringModelOutput]:
        r"""
        start_positions (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
            Labels for position (index) of the start of the labelled span for computing the token classification loss.
            Positions are clamped to the length of the sequence (`sequence_length`). Position outside of the sequence
            are not taken into account for computing the loss.
        end_positions (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
            Labels for position (index) of the end of the labelled span for computing the token classification loss.
            Positions are clamped to the length of the sequence (`sequence_length`). Position outside of the sequence
            are not taken into account for computing the loss.
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        outputs = self.transformer(
            input_ids,
            attention_mask=attention_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )

        sequence_output = outputs[0]

        logits = self.qa_outputs(sequence_output)
        start_logits, end_logits = logits.split(1, dim=-1)
        start_logits = start_logits.squeeze(-1).contiguous()
        end_logits = end_logits.squeeze(-1).contiguous()

        total_loss = None
        if start_positions is not None and end_positions is not None:
            # If we are on multi-GPU, split add a dimension
            if len(start_positions.size()) > 1:
                start_positions = start_positions.squeeze(-1)
            if len(end_positions.size()) > 1:
                end_positions = end_positions.squeeze(-1)
            # sometimes the start/end positions are outside our model inputs, we ignore these terms
            ignored_index = start_logits.size(1)
            start_positions = start_positions.clamp(0, ignored_index)
            end_positions = end_positions.clamp(0, ignored_index)

            loss_fct = CrossEntropyLoss(ignore_index=ignored_index)
            start_loss = loss_fct(start_logits, start_positions)
            end_loss = loss_fct(end_logits, end_positions)
            total_loss = (start_loss + end_loss) / 2

        if not return_dict:
            output = (start_logits, end_logits) + outputs[2:]
            return ((total_loss,) + output) if total_loss is not None else output

        return QuestionAnsweringModelOutput(
            loss=total_loss,
            start_logits=start_logits,
            end_logits=end_logits,
            hidden_states=outputs.hidden_states,
            attentions=outputs.attentions,
        )



# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..utils import DummyObject, requires_backends


class TensorFlowBenchmarkArguments(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TensorFlowBenchmark(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFForcedBOSTokenLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFForcedEOSTokenLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFForceTokensLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGenerationMixin(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLogitsProcessorList(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLogitsWarper(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMinLengthLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFNoBadWordsLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFNoRepeatNGramLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRepetitionPenaltyLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSuppressTokensAtBeginLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSuppressTokensLogitsProcessor(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTemperatureLogitsWarper(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTopKLogitsWarper(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTopPLogitsWarper(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def tf_top_k_top_p_filtering(*args, **kwargs):
    requires_backends(tf_top_k_top_p_filtering, ["tf"])


class KerasMetricCallback(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class PushToHubCallback(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSequenceSummary(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSharedEmbeddings(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def shape_list(*args, **kwargs):
    requires_backends(shape_list, ["tf"])


TF_ALBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFAlbertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MODEL_FOR_AUDIO_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_CAUSAL_LM_MAPPING = None


TF_MODEL_FOR_DOCUMENT_QUESTION_ANSWERING_MAPPING = None


TF_MODEL_FOR_IMAGE_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_MASK_GENERATION_MAPPING = None


TF_MODEL_FOR_MASKED_IMAGE_MODELING_MAPPING = None


TF_MODEL_FOR_MASKED_LM_MAPPING = None


TF_MODEL_FOR_MULTIPLE_CHOICE_MAPPING = None


TF_MODEL_FOR_NEXT_SENTENCE_PREDICTION_MAPPING = None


TF_MODEL_FOR_PRETRAINING_MAPPING = None


TF_MODEL_FOR_QUESTION_ANSWERING_MAPPING = None


TF_MODEL_FOR_SEMANTIC_SEGMENTATION_MAPPING = None


TF_MODEL_FOR_SEQ_TO_SEQ_CAUSAL_LM_MAPPING = None


TF_MODEL_FOR_SEQUENCE_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_SPEECH_SEQ_2_SEQ_MAPPING = None


TF_MODEL_FOR_TABLE_QUESTION_ANSWERING_MAPPING = None


TF_MODEL_FOR_TEXT_ENCODING_MAPPING = None


TF_MODEL_FOR_TOKEN_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_VISION_2_SEQ_MAPPING = None


TF_MODEL_FOR_ZERO_SHOT_IMAGE_CLASSIFICATION_MAPPING = None


TF_MODEL_MAPPING = None


TF_MODEL_WITH_LM_HEAD_MAPPING = None


class TFAutoModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForAudioClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForDocumentQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMaskedImageModeling(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMaskGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForNextSentencePrediction(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSemanticSegmentation(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSeq2SeqLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSpeechSeq2Seq(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForTableQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForTextEncoding(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForVision2Seq(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForZeroShotImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelWithLMHead(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartPretrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_BERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFBertEmbeddings(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForNextSentencePrediction(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_BLIP_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFBlipForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipForImageTextRetrieval(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipTextModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlipVisionModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCamembertForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CLIP_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCLIPModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCLIPPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCLIPTextModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCLIPVisionModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CONVBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFConvBertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvNextForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvNextModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvNextPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CTRL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCTRLForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CVT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCvtForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCvtModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCvtPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFData2VecVisionForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFData2VecVisionForSemanticSegmentation(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFData2VecVisionModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFData2VecVisionPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DEBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDebertaForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DEBERTA_V2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDebertaV2ForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2PreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DEIT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDeiTForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDeiTForImageClassificationWithTeacher(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDeiTForMaskedImageModeling(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDeiTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDeiTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DISTILBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDistilBertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DPR_CONTEXT_ENCODER_PRETRAINED_MODEL_ARCHIVE_LIST = None


TF_DPR_QUESTION_ENCODER_PRETRAINED_MODEL_ARCHIVE_LIST = None


TF_DPR_READER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDPRContextEncoder(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedContextEncoder(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedQuestionEncoder(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedReader(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRQuestionEncoder(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRReader(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_EFFICIENTFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFEfficientFormerForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEfficientFormerForImageClassificationWithTeacher(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEfficientFormerModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEfficientFormerPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ELECTRA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFElectraForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEncoderDecoderModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


ESM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFEsmForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEsmForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEsmForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEsmModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEsmPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_FLAUBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFFlaubertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForQuestionAnsweringSimple(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertWithLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_FUNNEL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFFunnelBaseModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_GPT2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFGPT2DoubleHeadsModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2ForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2LMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2MainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2PreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPTJForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPTJForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPTJForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPTJModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPTJPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_GROUPVIT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFGroupViTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGroupViTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGroupViTTextModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGroupViTVisionModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_HUBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFHubertForCTC(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFHubertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFHubertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LAYOUTLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLayoutLMForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LAYOUTLMV3_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLayoutLMv3ForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMv3ForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMv3ForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMv3Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMv3PreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LONGFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLongformerForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerSelfAttention(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LXMERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLxmertForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertVisualFeatureEncoder(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianMTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MOBILEBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFMobileBertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForNextSentencePrediction(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MOBILEVIT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFMobileViTForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileViTForSemanticSegmentation(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileViTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileViTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MPNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFMPNetForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5EncoderModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5ForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_OPENAI_GPT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFOpenAIGPTDoubleHeadsModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOPTForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOPTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOPTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagSequenceForGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagTokenForGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_REGNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRegNetForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRegNetModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRegNetPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_REMBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRemBertForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_RESNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFResNetForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFResNetModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFResNetPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRobertaForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ROBERTA_PRELAYERNORM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRobertaPreLayerNormForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreLayerNormPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ROFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRoFormerForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_SAM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFSamModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSamPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_SEGFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFSegformerDecodeHead(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSegformerForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSegformerForSemanticSegmentation(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSegformerModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSegformerPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_SPEECH_TO_TEXT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFSpeech2TextForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSpeech2TextModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSpeech2TextPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_SWIN_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFSwinForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSwinForMaskedImageModeling(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSwinModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSwinPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_T5_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFT5EncoderModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5ForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5PreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_TAPAS_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFTapasForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_TRANSFO_XL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFAdaptiveEmbedding(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFVisionEncoderDecoderModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFVisionTextDualEncoderModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTForImageClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTMAEForPreTraining(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTMAEModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTMAEPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_WAV_2_VEC_2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFWav2Vec2ForCTC(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWav2Vec2ForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWav2Vec2Model(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWav2Vec2PreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_WHISPER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFWhisperForConditionalGeneration(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWhisperModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWhisperPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XGLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXGLMForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXGLMModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXGLMPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLMForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForQuestionAnsweringSimple(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMWithLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLMRobertaForCausalLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForMaskedLM(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForQuestionAnswering(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLNetForMultipleChoice(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForQuestionAnsweringSimple(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForSequenceClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForTokenClassification(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetLMHeadModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetMainLayer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetPreTrainedModel(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class AdamWeightDecay(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class GradientAccumulator(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class WarmUp(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def create_optimizer(*args, **kwargs):
    requires_backends(create_optimizer, ["tf"])


class TFTrainer(metaclass=DummyObject):
    _backends = ["tf"]

    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])



# coding=utf-8
# Copyright 2022 The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Convert data2vec checkpoint."""


import argparse
import os
import pathlib

import fairseq
import torch
from fairseq.modules import TransformerSentenceEncoderLayer
from packaging import version

from transformers import (
    Data2VecTextConfig,
    Data2VecTextForMaskedLM,
    Data2VecTextForSequenceClassification,
    Data2VecTextModel,
)
from transformers.models.bert.modeling_bert import (
    BertIntermediate,
    BertLayer,
    BertOutput,
    BertSelfAttention,
    BertSelfOutput,
)

# IMPORTANT: In order for this script to run, please make sure to download the dictionary: `dict.txt` from wget https://dl.fbaipublicfiles.com/fairseq/models/roberta.large.tar.gz
# File copied from https://github.com/pytorch/fairseq/blob/main/examples/data2vec/models/data2vec_text.py
from transformers.utils import logging


if version.parse(fairseq.__version__) < version.parse("0.9.0"):
    raise Exception("requires fairseq >= 0.9.0")


logging.set_verbosity_info()
logger = logging.get_logger(__name__)

SAMPLE_TEXT = "Hello world! cécé herlolip"


def convert_data2vec_checkpoint_to_pytorch(
    data2vec_checkpoint_path: str, pytorch_dump_folder_path: str, classification_head: bool
):
    """
    Copy/paste/tweak data2vec's weights to our BERT structure.
    """
    data2vec_checkpoint_dir, data2vec_checkpoint_file_name = os.path.split(data2vec_checkpoint_path)
    data2vec = Data2VecTextModel.from_pretrained(
        data2vec_checkpoint_dir, checkpoint_file=data2vec_checkpoint_file_name
    )
    data2vec.eval()  # disable dropout
    data2vec_model = data2vec.models[0]
    data2vec_sent_encoder = data2vec_model.encoder.sentence_encoder
    config = Data2VecTextConfig(
        vocab_size=data2vec_sent_encoder.embed_tokens.num_embeddings,
        hidden_size=data2vec_model.args.encoder_embed_dim,
        num_hidden_layers=data2vec_model.args.encoder_layers,
        num_attention_heads=data2vec_model.args.encoder_attention_heads,
        intermediate_size=data2vec_model.args.encoder_ffn_embed_dim,
        max_position_embeddings=514,
        type_vocab_size=1,
        layer_norm_eps=1e-5,  # PyTorch default used in fairseq
    )
    if classification_head:
        config.num_labels = data2vec.model.classification_heads["mnli"].out_proj.weight.shape[0]
    print("Our BERT config:", config)

    model = Data2VecTextForSequenceClassification(config) if classification_head else Data2VecTextForMaskedLM(config)
    model.eval()

    # Now let's copy all the weights.
    # Embeddings
    model.data2vec_text.embeddings.word_embeddings.weight = data2vec_sent_encoder.embed_tokens.weight
    model.data2vec_text.embeddings.position_embeddings.weight = data2vec_sent_encoder.embed_positions.weight
    model.data2vec_text.embeddings.token_type_embeddings.weight.data = torch.zeros_like(
        model.data2vec_text.embeddings.token_type_embeddings.weight
    )  # just zero them out b/c data2vec doesn't use them.
    model.data2vec_text.embeddings.LayerNorm.weight = data2vec_sent_encoder.layernorm_embedding.weight
    model.data2vec_text.embeddings.LayerNorm.bias = data2vec_sent_encoder.layernorm_embedding.bias

    for i in range(config.num_hidden_layers):
        # Encoder: start of layer
        layer: BertLayer = model.data2vec_text.encoder.layer[i]
        data2vec_layer: TransformerSentenceEncoderLayer = data2vec_sent_encoder.layers[i]

        # self attention
        self_attn: BertSelfAttention = layer.attention.self
        assert data2vec_layer.self_attn.k_proj.weight.data.shape == torch.Size(
            (config.hidden_size, config.hidden_size)
        ), (
            "Shape for data2vec_layer.self_attn.k_proj.weight.data should be"
            f" {torch.Size((config.hidden_size, config.hidden_size))}"
        )
        assert data2vec_layer.self_attn.q_proj.weight.data.shape == torch.Size(
            (config.hidden_size, config.hidden_size)
        ), (
            "Shape for data2vec_layer.self_attn.q_proj.weight.data should be"
            f" {torch.Size((config.hidden_size, config.hidden_size))}"
        )
        assert data2vec_layer.self_attn.v_proj.weight.data.shape == torch.Size(
            (config.hidden_size, config.hidden_size)
        ), (
            "Shape for data2vec_layer.self_attn.v_proj.weight.data should be"
            f" {torch.Size((config.hidden_size, config.hidden_size))}"
        )

        self_attn.query.weight.data = data2vec_layer.self_attn.q_proj.weight
        self_attn.query.bias.data = data2vec_layer.self_attn.q_proj.bias
        self_attn.key.weight.data = data2vec_layer.self_attn.k_proj.weight
        self_attn.key.bias.data = data2vec_layer.self_attn.k_proj.bias
        self_attn.value.weight.data = data2vec_layer.self_attn.v_proj.weight
        self_attn.value.bias.data = data2vec_layer.self_attn.v_proj.bias

        # self-attention output
        self_output: BertSelfOutput = layer.attention.output
        assert (
            self_output.dense.weight.shape == data2vec_layer.self_attn.out_proj.weight.shape
        ), f"Shape for self_output.dense.weight should be {data2vec_layer.self_attn.out_proj.weight.shape}"
        self_output.dense.weight = data2vec_layer.self_attn.out_proj.weight
        self_output.dense.bias = data2vec_layer.self_attn.out_proj.bias
        self_output.LayerNorm.weight = data2vec_layer.self_attn_layer_norm.weight
        self_output.LayerNorm.bias = data2vec_layer.self_attn_layer_norm.bias

        # intermediate
        intermediate: BertIntermediate = layer.intermediate
        assert (
            intermediate.dense.weight.shape == data2vec_layer.fc1.weight.shape
        ), f"Shape for intermediate.dense.weight should be {data2vec_layer.fc1.weight.shape}"
        intermediate.dense.weight = data2vec_layer.fc1.weight
        intermediate.dense.bias = data2vec_layer.fc1.bias

        # output
        bert_output: BertOutput = layer.output
        assert (
            bert_output.dense.weight.shape == data2vec_layer.fc2.weight.shape
        ), f"Shape for bert_output.dense.weight should be {data2vec_layer.fc2.weight.shape}"
        bert_output.dense.weight = data2vec_layer.fc2.weight
        bert_output.dense.bias = data2vec_layer.fc2.bias
        bert_output.LayerNorm.weight = data2vec_layer.final_layer_norm.weight
        bert_output.LayerNorm.bias = data2vec_layer.final_layer_norm.bias
        # end of layer

    if classification_head:
        model.classifier.dense.weight = data2vec.model.classification_heads["mnli"].dense.weight
        model.classifier.dense.bias = data2vec.model.classification_heads["mnli"].dense.bias
        model.classifier.out_proj.weight = data2vec.model.classification_heads["mnli"].out_proj.weight
        model.classifier.out_proj.bias = data2vec.model.classification_heads["mnli"].out_proj.bias
    else:
        # LM Head
        model.lm_head.dense.weight = data2vec_model.encoder.lm_head.dense.weight
        model.lm_head.dense.bias = data2vec_model.encoder.lm_head.dense.bias
        model.lm_head.layer_norm.weight = data2vec_model.encoder.lm_head.layer_norm.weight
        model.lm_head.layer_norm.bias = data2vec_model.encoder.lm_head.layer_norm.bias
        model.lm_head.decoder.weight = data2vec_model.encoder.lm_head.weight
        model.lm_head.decoder.bias = data2vec_model.encoder.lm_head.bias

    # Let's check that we get the same results.
    input_ids: torch.Tensor = data2vec.encode(SAMPLE_TEXT).unsqueeze(0)  # batch of size 1

    our_output = model(input_ids)[0]
    if classification_head:
        their_output = data2vec.model.classification_heads["mnli"](data2vec.extract_features(input_ids))
    else:
        their_output = data2vec_model(input_ids)[0]
    print(our_output.shape, their_output.shape)
    max_absolute_diff = torch.max(torch.abs(our_output - their_output)).item()
    print(f"max_absolute_diff = {max_absolute_diff}")  # ~ 1e-7
    success = torch.allclose(our_output, their_output, atol=1e-3)
    print("Do both models output the same tensors?", "🔥" if success else "💩")
    if not success:
        raise Exception("Something went wRoNg")

    pathlib.Path(pytorch_dump_folder_path).mkdir(parents=True, exist_ok=True)
    print(f"Saving model to {pytorch_dump_folder_path}")
    model.save_pretrained(pytorch_dump_folder_path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # Required parameters
    parser.add_argument(
        "--checkpoint_path", default=None, type=str, required=True, help="Path the official PyTorch dump."
    )
    parser.add_argument(
        "--pytorch_dump_folder_path", default=None, type=str, required=True, help="Path to the output PyTorch model."
    )
    parser.add_argument(
        "--classification_head", action="store_true", help="Whether to convert a final classification head."
    )
    args = parser.parse_args()
    convert_data2vec_checkpoint_to_pytorch(
        args.checkpoint_path, args.pytorch_dump_folder_path, args.classification_head
    )



print("import: 'transformers'")
import transformers

print("import: 'transformers.benchmark'")
import transformers.benchmark

print("import: 'transformers.commands'")
import transformers.commands

print("import: 'transformers.data'")
import transformers.data

print("import: 'transformers.kernels'")
import transformers.kernels

print("import: 'transformers.models'")
import transformers.models

print("import: 'transformers.pipelines'")
import transformers.pipelines

print("import: 'transformers.tools'")
import transformers.tools

print("import: 'transformers.utils'")
import transformers.utils




# coding=utf-8
# Copyright 2020 The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Convert ConvBERT checkpoint."""

import argparse

from transformers import ConvBertConfig, ConvBertModel, TFConvBertModel, load_tf_weights_in_convbert
from transformers.utils import logging


logging.set_verbosity_info()


def convert_orig_tf1_checkpoint_to_pytorch(tf_checkpoint_path, convbert_config_file, pytorch_dump_path):
    conf = ConvBertConfig.from_json_file(convbert_config_file)
    model = ConvBertModel(conf)

    model = load_tf_weights_in_convbert(model, conf, tf_checkpoint_path)
    model.save_pretrained(pytorch_dump_path)

    tf_model = TFConvBertModel.from_pretrained(pytorch_dump_path, from_pt=True)
    tf_model.save_pretrained(pytorch_dump_path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # Required parameters
    parser.add_argument(
        "--tf_checkpoint_path", default=None, type=str, required=True, help="Path to the TensorFlow checkpoint path."
    )
    parser.add_argument(
        "--convbert_config_file",
        default=None,
        type=str,
        required=True,
        help=(
            "The config json file corresponding to the pre-trained ConvBERT model. \n"
            "This specifies the model architecture."
        ),
    )
    parser.add_argument(
        "--pytorch_dump_path", default=None, type=str, required=True, help="Path to the output PyTorch model."
    )
    args = parser.parse_args()
    convert_orig_tf1_checkpoint_to_pytorch(args.tf_checkpoint_path, args.convbert_config_file, args.pytorch_dump_path)



# coding=utf-8
# Copyright 2022 Salesforce authors, The EleutherAI, and HuggingFace Teams. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" CodeGen model configuration"""
from collections import OrderedDict
from typing import Any, List, Mapping, Optional

from ... import PreTrainedTokenizer, TensorType, is_torch_available
from ...configuration_utils import PretrainedConfig
from ...onnx import OnnxConfigWithPast, PatchingSpec
from ...utils import logging


logger = logging.get_logger(__name__)


CODEGEN_PRETRAINED_CONFIG_ARCHIVE_MAP = {
    "Salesforce/codegen-350M-nl": "https://huggingface.co/Salesforce/codegen-350M-nl/resolve/main/config.json",
    "Salesforce/codegen-350M-multi": "https://huggingface.co/Salesforce/codegen-350M-multi/resolve/main/config.json",
    "Salesforce/codegen-350M-mono": "https://huggingface.co/Salesforce/codegen-350M-mono/resolve/main/config.json",
    "Salesforce/codegen-2B-nl": "https://huggingface.co/Salesforce/codegen-2B-nl/resolve/main/config.json",
    "Salesforce/codegen-2B-multi": "https://huggingface.co/Salesforce/codegen-2B-multi/resolve/main/config.json",
    "Salesforce/codegen-2B-mono": "https://huggingface.co/Salesforce/codegen-2B-mono/resolve/main/config.json",
    "Salesforce/codegen-6B-nl": "https://huggingface.co/Salesforce/codegen-6B-nl/resolve/main/config.json",
    "Salesforce/codegen-6B-multi": "https://huggingface.co/Salesforce/codegen-6B-multi/resolve/main/config.json",
    "Salesforce/codegen-6B-mono": "https://huggingface.co/Salesforce/codegen-6B-mono/resolve/main/config.json",
    "Salesforce/codegen-16B-nl": "https://huggingface.co/Salesforce/codegen-16B-nl/resolve/main/config.json",
    "Salesforce/codegen-16B-multi": "https://huggingface.co/Salesforce/codegen-16B-multi/resolve/main/config.json",
    "Salesforce/codegen-16B-mono": "https://huggingface.co/Salesforce/codegen-16B-mono/resolve/main/config.json",
}


class CodeGenConfig(PretrainedConfig):
    r"""
    This is the configuration class to store the configuration of a [`CodeGenModel`]. It is used to instantiate a
    CodeGen model according to the specified arguments, defining the model architecture. Instantiating a configuration
    with the defaults will yield a similar configuration to that of the CodeGen
    [Salesforce/codegen-2B-mono](https://huggingface.co/Salesforce/codegen-2B-mono) architecture. Configuration objects
    inherit from [`PretrainedConfig`] and can be used to control the model outputs. Read the documentation from
    [`PretrainedConfig`] for more information.

    Args:
        vocab_size (`int`, *optional*, defaults to 50400):
            Vocabulary size of the CodeGen model. Defines the number of different tokens that can be represented by the
            `inputs_ids` passed when calling [`CodeGenModel`].
        n_positions (`int`, *optional*, defaults to 2048):
            The maximum sequence length that this model might ever be used with. Typically set this to something large
            just in case (e.g., 512 or 1024 or 2048).
        n_embd (`int`, *optional*, defaults to 4096):
            Dimensionality of the embeddings and hidden states.
        n_layer (`int`, *optional*, defaults to 28):
            Number of hidden layers in the Transformer encoder.
        n_head (`int`, *optional*, defaults to 16):
            Number of attention heads for each attention layer in the Transformer encoder.
        rotary_dim (`int`, *optional*, defaults to 64):
            Number of dimensions in the embedding that Rotary Position Embedding is applied to.
        n_inner (`int`, *optional*, defaults to None):
            Dimensionality of the inner feed-forward layers. `None` will set it to 4 times n_embd
        activation_function (`str`, *optional*, defaults to `"gelu_new"`):
            Activation function, to be selected in the list `["relu", "silu", "gelu", "tanh", "gelu_new"]`.
        resid_pdrop (`float`, *optional*, defaults to 0.1):
            The dropout probability for all fully connected layers in the embeddings, encoder, and pooler.
        embd_pdrop (`int`, *optional*, defaults to 0.1):
            The dropout ratio for the embeddings.
        attn_pdrop (`float`, *optional*, defaults to 0.1):
            The dropout ratio for the attention.
        layer_norm_epsilon (`float`, *optional*, defaults to 1e-5):
            The epsilon to use in the layer normalization layers.
        initializer_range (`float`, *optional*, defaults to 0.02):
            The standard deviation of the truncated_normal_initializer for initializing all weight matrices.
        use_cache (`bool`, *optional*, defaults to `True`):
            Whether or not the model should return the last key/values attentions (not used by all models).

    Example:

    ```python
    >>> from transformers import CodeGenConfig, CodeGenModel

    >>> # Initializing a CodeGen 6B configuration
    >>> configuration = CodeGenConfig()

    >>> # Initializing a model (with random weights) from the configuration
    >>> model = CodeGenModel(configuration)

    >>> # Accessing the model configuration
    >>> configuration = model.config
    ```"""
    model_type = "codegen"
    attribute_map = {
        "max_position_embeddings": "n_positions",
        "hidden_size": "n_embd",
        "num_attention_heads": "n_head",
        "num_hidden_layers": "n_layer",
    }

    def __init__(
        self,
        vocab_size=50400,
        n_positions=2048,
        n_ctx=2048,
        n_embd=4096,
        n_layer=28,
        n_head=16,
        rotary_dim=64,
        n_inner=None,
        activation_function="gelu_new",
        resid_pdrop=0.0,
        embd_pdrop=0.0,
        attn_pdrop=0.0,
        layer_norm_epsilon=1e-5,
        initializer_range=0.02,
        use_cache=True,
        bos_token_id=50256,
        eos_token_id=50256,
        tie_word_embeddings=False,
        **kwargs,
    ):
        self.vocab_size = vocab_size
        self.n_ctx = n_ctx
        self.n_positions = n_positions
        self.n_embd = n_embd
        self.n_layer = n_layer
        self.n_head = n_head
        self.n_inner = n_inner
        self.rotary_dim = rotary_dim
        self.activation_function = activation_function
        self.resid_pdrop = resid_pdrop
        self.embd_pdrop = embd_pdrop
        self.attn_pdrop = attn_pdrop
        self.layer_norm_epsilon = layer_norm_epsilon
        self.initializer_range = initializer_range
        self.use_cache = use_cache

        self.bos_token_id = bos_token_id
        self.eos_token_id = eos_token_id

        super().__init__(
            bos_token_id=bos_token_id, eos_token_id=eos_token_id, tie_word_embeddings=tie_word_embeddings, **kwargs
        )


# Copied from transformers.models.gpt2.configuration_gpt2.GPT2OnnxConfig
class CodeGenOnnxConfig(OnnxConfigWithPast):
    def __init__(
        self,
        config: PretrainedConfig,
        task: str = "default",
        patching_specs: List[PatchingSpec] = None,
        use_past: bool = False,
    ):
        super().__init__(config, task=task, patching_specs=patching_specs, use_past=use_past)
        if not getattr(self._config, "pad_token_id", None):
            # TODO: how to do that better?
            self._config.pad_token_id = 0

    @property
    def inputs(self) -> Mapping[str, Mapping[int, str]]:
        common_inputs = OrderedDict({"input_ids": {0: "batch", 1: "sequence"}})
        if self.use_past:
            self.fill_with_past_key_values_(common_inputs, direction="inputs")
            common_inputs["attention_mask"] = {0: "batch", 1: "past_sequence + sequence"}
        else:
            common_inputs["attention_mask"] = {0: "batch", 1: "sequence"}

        return common_inputs

    @property
    def num_layers(self) -> int:
        return self._config.n_layer

    @property
    def num_attention_heads(self) -> int:
        return self._config.n_head

    def generate_dummy_inputs(
        self,
        tokenizer: PreTrainedTokenizer,
        batch_size: int = -1,
        seq_length: int = -1,
        is_pair: bool = False,
        framework: Optional[TensorType] = None,
    ) -> Mapping[str, Any]:
        common_inputs = super(OnnxConfigWithPast, self).generate_dummy_inputs(
            tokenizer, batch_size=batch_size, seq_length=seq_length, is_pair=is_pair, framework=framework
        )

        # We need to order the input in the way they appears in the forward()
        ordered_inputs = OrderedDict({"input_ids": common_inputs["input_ids"]})

        # Need to add the past_keys
        if self.use_past:
            if not is_torch_available():
                raise ValueError("Cannot generate dummy past_keys inputs without PyTorch installed.")
            else:
                import torch

                batch, seqlen = common_inputs["input_ids"].shape
                # Not using the same length for past_key_values
                past_key_values_length = seqlen + 2
                past_shape = (
                    batch,
                    self.num_attention_heads,
                    past_key_values_length,
                    self._config.hidden_size // self.num_attention_heads,
                )
                ordered_inputs["past_key_values"] = [
                    (torch.zeros(past_shape), torch.zeros(past_shape)) for _ in range(self.num_layers)
                ]

        ordered_inputs["attention_mask"] = common_inputs["attention_mask"]
        if self.use_past:
            mask_dtype = ordered_inputs["attention_mask"].dtype
            ordered_inputs["attention_mask"] = torch.cat(
                [ordered_inputs["attention_mask"], torch.ones(batch, past_key_values_length, dtype=mask_dtype)], dim=1
            )

        return ordered_inputs

    @property
    def default_onnx_opset(self) -> int:
        return 13
